import logging
import socket
import sys

from chatclient.сhatclient.Mainlib.variables import ACTION, PRESENCE

if sys.argv[0].find('client') == -1:
	# если не клиент то сервер!
	server_log = logging.getLogger('server-log')
	print(server_log.getEffectiveLevel())
else:
	client_log = logging.getLogger('client-log')
	print(client_log.getEffectiveLevel())


def log(func_to_log):
	"""
	A decorator that logs function calls.
    Saves events containing
    information about the name of the called function, the parameters with which
    the function is called, and the module is calling the function.
    """
	
	def log_saver(*args, **kwargs):
		ret = func_to_log(*args, **kwargs)
		client_log.debug(f'Была вызвана функция {func_to_log.__name__} c параметрами {args}, {kwargs}. '
						 f'Вызов из модуля {func_to_log.__module__}')
		return ret
	
	return log_saver


def login_required(func):
	"""
	A decorator that verifies that the client is authorized on the server.
	Checks that the passed socket object is in
	the list of authorized clients.
	Except for passing the dictionary-query
	for authorization. If the client is not authorized,
	throws a TypeError exception
	"""
	
	def checker(*args, **kwargs):
		# проверяем, что первый аргумент - экземпляр MessageProcessor
		# Импортить необходимо тут, иначе ошибка рекурсивного импорта.
		from chatserver.сhatserver.server.server_core import MessageProcessor
		if isinstance(args[0], MessageProcessor):
			found = False
			for arg in args:
				if isinstance(arg, socket.socket):
					# Проверяем, что данный сокет есть в списке names класса
					# MessageProcessor
					for client in args[0].names:
						if args[0].names[client] == arg:
							found = True
			
			# Теперь надо проверить, что передаваемые аргументы не presence
			# сообщение. Если presense, то разрешаем
			for arg in args:
				if isinstance(arg, dict):
					if ACTION in arg and arg[ACTION] == PRESENCE:
						found = True
			# Если не не авторизован и не сообщение начала авторизации, то
			# вызываем исключение.
			if not found:
				raise TypeError
		return func(*args, **kwargs)
	
	return checker



import json

from chatclient.сhatclient.Log.Log_decorators import log


class JIM(object):
    pass


@log
def get_message(client):
    """
    The utility for receiving and decoding a message accepts bytes and gives out a dictionary,
    if something else is received it gives a value error
    :param client:
    :return: dict
    """
    encoded_response = client.recv(10240)
    if isinstance(encoded_response, bytes):
        json_response = encoded_response.decode('utf-8')
        response = json.loads(json_response)
        if isinstance(response, dict):
            return response
        else:
            raise Exception('Decoding error')

    else:
        raise Exception


@log
def send_message(sock, message):
    """
    Message encoding and sending utility,
    takes a dictionary and sends it
    :param sock:
    :param message:
    :return: bytes
    """
    if not isinstance(message, dict):
        raise Exception('Encoding error')
    js_message = json.dumps(message)
    encoded_message = js_message.encode('utf-8')
    sock.send(encoded_message)


class JimResponse(JIM):

    def status_200(self):
        msg = {
            "response": 200,
            "alert": "Необязательное сообщение/уведомление"
        }
        print(msg)
        return msg

    def status_400(self):
        msg = {
            "response": 400,
            "error": "Bad Request"
        }
        print(msg)
        return msg

    def status_401(self):
        msg = {
            "response": 401,
            "error": "Name is already taken"
        }
        print(msg)
        return msg

    def status_402(self):
        msg = {
            "response": 402,
            "error": "This could be wrong password or no account with that name"
        }
        print(msg)
        return msg



import ipaddress

from chatclient.сhatclient.Log.client_log_config import client_log


class Port:
    """
    For port number descriptor: Validating the port.

    """

    def __set__(self, instance, value):
        if not 1023 < value < 65536:
            client_log.critical(
                f'Недопустимый порт {value}. Допустимы порты с 1024 до 65535.')
            exit(1)
        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name


class IPAddress:
    """
    The descriptor for the IP address: Validating the IP address
    """

    def __set__(self, instance, value):
        try:
            if value == str(ipaddress.ip_address(value)):
                instance.__dict__[self.name] = value
        except ValueError:
            client_log.critical(f'{value} не является корректным IP-адресом')
            raise Exception(f'{value} не является корректным IP-адресом')

    def __set_name__(self, owner, name):
        self.name = name



import datetime
import os

from sqlalchemy import create_engine, Column, Integer, String, Text, DateTime
from sqlalchemy.exc import DisconnectionError
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
from sqlalchemy.sql import default_comparator

# Класс - база данных клиента.
from chatclient.сhatclient.Log.client_log_config import client_log


class ClientDatabase:
	"""
    The class is a wrapper for working with the client database.
    Uses SQLite database, implemented with
    SQLAlchemy ORM and use a declarative approach
    """
	CDB_Base = declarative_base()
	
	class KnownUsers(CDB_Base):
		"""
        Class displaying all known users
        """
		__tablename__ = 'known_users'
		id = Column(Integer, primary_key=True)
		username = Column(String)
		
		def __init__(self, user):
			self.id = None
			self.username = user
	
	class MessageHistory(CDB_Base):
		"""
        Class displaying message history
        """
		__tablename__ = 'message_history'
		id = Column(Integer, primary_key=True)
		contact = Column(String)
		direction = Column(String)
		message = Column(Text)
		date = Column(DateTime)
		
		def __init__(self, contact, direction, message):
			self.id = None
			self.contact = contact
			self.direction = direction
			self.message = message
			self.date = datetime.datetime.now()
	
	# Класс - отображение списка контактов
	class Contacts(CDB_Base):
		"""
        Class displaying a list of contacts
        """
		__tablename__ = 'contacts'
		id = Column(Integer, primary_key=True)
		name = Column(String, unique=True)
		
		def __init__(self, contact):
			self.id = None
			self.name = contact
	
	def __init__(self, name):
		# Создаём движок базы данных, поскольку разрешено несколько клиентов одновременно, каждый должен иметь свою БД
		# Поскольку клиент мультипоточный необходимо отключить проверки на подключения с разных потоков,
		# иначе sqlite3.ProgrammingError
		path = os.getcwd()
		filename = f'client_{name}.db3'
		self.database_engine = create_engine(f'sqlite:///{os.path.join(path, filename)}',
											 echo=False,
											 pool_recycle=7200,
											 connect_args={
												 'check_same_thread': False})
		
		self.CDB_Base.metadata.create_all(self.database_engine)
		
		# Создаём сессию
		Session = sessionmaker(bind=self.database_engine)
		self.session = Session()
		self.session.commit()
		
		# Необходимо очистить таблицу контактов, т.к. при запуске они подгружаются с сервера.
		self.session.query(self.Contacts).delete()
		self.session.commit()
	
	def check_connection(self):
		"""
        Checking the database connection
        :return:
        """
		global conn_db
		result = False
		try:
			conn_db = self.database_engine.connect()
			result = True
		except DisconnectionError:
			client_log.critical(f'Нет соединения с базой данных. Test connection {result}')
			print(f'Нет соединения с базой данных. Test connection {result}')
		finally:
			conn_db.close()
		print(f'Test connection {result}')
		return result
	
	def add_contact(self, contact):
		"""
        Add contacts function
        :param contact:
        :return:
        """
		if not self.session.query(self.Contacts).filter_by(name=contact).count():
			contact_row = self.Contacts(contact)
			self.session.add(contact_row)
			self.session.commit()
	
	def del_contact(self, contact):
		"""
        Contact delete function
        :param contact:
        :return:
        """
		self.session.query(self.Contacts).filter_by(name=contact).delete()
	
	def contacts_clear(self):
		"""
		Method for clearing a table with a contact list
		"""
		self.session.query(self.Contacts).delete()
	
	def add_users(self, users_list):
		"""
        Feature of adding famous users. Users are retrieved only from the server, so the table is cleared
        :param users_list:
        :return:
        """
		self.session.query(self.KnownUsers).delete()
		for user in users_list:
			user_row = self.KnownUsers(user)
			self.session.add(user_row)
		self.session.commit()
	
	def save_message(self, contact, direction, message):
		"""
        Save message function
        :param contact:
        :param direction:
        :param message:
        :return:
        """
		message_row = self.MessageHistory(contact, direction, message)
		self.session.add(message_row)
		self.session.commit()
	
	def get_contacts(self):
		"""
        Returning contacts function
        :return:
        """
		return [contact[0] for contact in self.session.query(self.Contacts.name).all()]
	
	def get_users(self):
		"""
        Function that returns a list of known users
        :return:
        """
		return [user[0] for user in self.session.query(self.KnownUsers.username).all()]
	
	def check_user(self, user):
		"""
        A function that checks the presence of a user in known
        :param user:
        :return:
        """
		if self.session.query(self.KnownUsers).filter_by(username=user).count():
			return True
		else:
			return False
	
	def check_contact(self, contact):
		"""
        Function that checks the presence of a user in contacts
        :param contact:
        :return:
        """
		if self.session.query(self.Contacts).filter_by(name=contact).count():
			return True
		else:
			return False
	
	def get_history(self, contact):
		"""
        Function that returns the conversation history
        :param contact: this field from
        :return:
        """
		query = self.session.query(
			self.MessageHistory).filter_by(
			contact=contact)
		return [(history_row.contact, history_row.direction, history_row.message, history_row.date)
				for history_row in query.all()]


if __name__ == '__main__':
	test_db = ClientDatabase('test55')
	# for i in ['test3', 'test4', 'test5']:
	#     test_db.add_contact(i)
	# test_db.check_connection()
	# test_db.add_contact('test4')
	# test_db.add_users(['test1', 'test2', 'test3', 'test4', 'test5'])
	# test_db.save_message('test1', 'test2', f'Привет! я тестовое сообщение от {datetime.datetime.now()}!')
	# test_db.save_message('test2', 'test1', f'Привет! я другое тестовое сообщение от {datetime.datetime.now()}!')
	# print(test_db.get_contacts())
	# print(test_db.get_users())
	# print(test_db.check_user('test1'))
	# print(test_db.check_user('test10'))
	# print(test_db.get_history('test2'))
	# print(test_db.get_history(contact='test2'))
	# print(test_db.get_history('test3'))
	# test_db.del_contact('test4')
	# print(test_db.get_contacts())



class IncorrectDataRecivedError(Exception):
    """
    Exception - Invalid data received from socket
    """

    def __str__(self):
        return 'Принято некорректное сообщение от удалённого компьютера.'


class ServerError(Exception):
    """
    Exception - Server Error
    """

    def __init__(self, text):
        self.text = text

    def __str__(self):
        return self.text


class NonDictInputError(Exception):
    """
    Exception - is the function argument is not a dictionary.
    """

    def __str__(self):
        return 'Аргумент функции должен быть словарём.'


class ReqFieldMissingError(Exception):
    """
    Error - a required field is missing in the accepted dictionary
    """

    def __init__(self, missing_field):
        self.missing_field = missing_field

    def __str__(self):
        return f'В принятом словаре отсутствует обязательное поле {self.missing_field}.'


class DataBaseInteractiveError(Exception):
    """
    Database communication error
    """

    def __str__(self):
        return 'Ошибка взаимодействия с базой данных'



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'client.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtWidgets


class Ui_MainClientWindow(object):
	"""
	Class is the main user window.
    Contains all the main logic of the client module.
    The converted file. Loaded from 'client.ui'
	"""
	def setupUi(self, MainClientWindow):
		MainClientWindow.setObjectName("MainClientWindow")
		MainClientWindow.resize(756, 534)
		MainClientWindow.setMinimumSize(QtCore.QSize(756, 534))
		self.centralwidget = QtWidgets.QWidget(MainClientWindow)
		self.centralwidget.setObjectName("centralwidget")
		self.label_contacts = QtWidgets.QLabel(self.centralwidget)
		self.label_contacts.setGeometry(QtCore.QRect(10, 0, 101, 16))
		self.label_contacts.setObjectName("label_contacts")
		self.btn_add_contact = QtWidgets.QPushButton(self.centralwidget)
		self.btn_add_contact.setGeometry(QtCore.QRect(10, 450, 121, 31))
		self.btn_add_contact.setObjectName("btn_add_contact")
		self.btn_remove_contact = QtWidgets.QPushButton(self.centralwidget)
		self.btn_remove_contact.setGeometry(QtCore.QRect(140, 450, 121, 31))
		self.btn_remove_contact.setObjectName("btn_remove_contact")
		self.label_history = QtWidgets.QLabel(self.centralwidget)
		self.label_history.setGeometry(QtCore.QRect(300, 0, 391, 21))
		self.label_history.setObjectName("label_history")
		self.text_message = QtWidgets.QTextEdit(self.centralwidget)
		self.text_message.setGeometry(QtCore.QRect(300, 360, 441, 71))
		self.text_message.setObjectName("text_message")
		self.label_new_message = QtWidgets.QLabel(self.centralwidget)
		self.label_new_message.setGeometry(QtCore.QRect(300, 330, 450, 16))  # Правка тут
		self.label_new_message.setObjectName("label_new_message")
		self.list_contacts = QtWidgets.QListView(self.centralwidget)
		self.list_contacts.setGeometry(QtCore.QRect(10, 20, 251, 411))
		self.list_contacts.setObjectName("list_contacts")
		self.list_messages = QtWidgets.QListView(self.centralwidget)
		self.list_messages.setGeometry(QtCore.QRect(300, 20, 441, 301))
		self.list_messages.setObjectName("list_messages")
		self.btn_send = QtWidgets.QPushButton(self.centralwidget)
		self.btn_send.setGeometry(QtCore.QRect(610, 450, 131, 31))
		self.btn_send.setObjectName("btn_send")
		self.btn_clear = QtWidgets.QPushButton(self.centralwidget)
		self.btn_clear.setGeometry(QtCore.QRect(460, 450, 131, 31))
		self.btn_clear.setObjectName("btn_clear")
		MainClientWindow.setCentralWidget(self.centralwidget)
		self.menubar = QtWidgets.QMenuBar(MainClientWindow)
		self.menubar.setGeometry(QtCore.QRect(0, 0, 756, 21))
		self.menubar.setObjectName("menubar")
		self.menu = QtWidgets.QMenu(self.menubar)
		self.menu.setObjectName("menu")
		self.menu_2 = QtWidgets.QMenu(self.menubar)
		self.menu_2.setObjectName("menu_2")
		MainClientWindow.setMenuBar(self.menubar)
		self.statusBar = QtWidgets.QStatusBar(MainClientWindow)
		self.statusBar.setObjectName("statusBar")
		MainClientWindow.setStatusBar(self.statusBar)
		self.menu_exit = QtWidgets.QAction(MainClientWindow)
		self.menu_exit.setObjectName("menu_exit")
		self.menu_add_contact = QtWidgets.QAction(MainClientWindow)
		self.menu_add_contact.setObjectName("menu_add_contact")
		self.menu_del_contact = QtWidgets.QAction(MainClientWindow)
		self.menu_del_contact.setObjectName("menu_del_contact")
		self.menu.addAction(self.menu_exit)
		self.menu_2.addAction(self.menu_add_contact)
		self.menu_2.addAction(self.menu_del_contact)
		self.menu_2.addSeparator()
		self.menubar.addAction(self.menu.menuAction())
		self.menubar.addAction(self.menu_2.menuAction())
		
		self.retranslateUi(MainClientWindow)
		self.btn_clear.clicked.connect(self.text_message.clear)
		QtCore.QMetaObject.connectSlotsByName(MainClientWindow)
	
	def retranslateUi(self, MainClientWindow):
		_translate = QtCore.QCoreApplication.translate
		MainClientWindow.setWindowTitle(_translate("MainClientWindow", "Чат Программа alpha release"))
		self.label_contacts.setText(_translate("MainClientWindow", "Список контактов:"))
		self.btn_add_contact.setText(_translate("MainClientWindow", "Добавить контакт"))
		self.btn_remove_contact.setText(_translate("MainClientWindow", "Удалить контакт"))
		self.label_history.setText(_translate("MainClientWindow", "История сообщений:"))
		self.label_new_message.setText(_translate("MainClientWindow", "Введите новое сообщение:"))
		self.btn_send.setText(_translate("MainClientWindow", "Отправить сообщение"))
		self.btn_clear.setText(_translate("MainClientWindow", "Очистить поле"))
		self.menu.setTitle(_translate("MainClientWindow", "Файл"))
		self.menu_2.setTitle(_translate("MainClientWindow", "Контакты"))
		self.menu_exit.setText(_translate("MainClientWindow", "Выход"))
		self.menu_add_contact.setText(_translate("MainClientWindow", "Добавить контакт"))
		self.menu_del_contact.setText(_translate("MainClientWindow", "Удалить контакт"))




from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QDialog, QLabel, QComboBox, QPushButton

from chatclient.сhatclient.Log.client_log_config import client_log


class AddContactDialog(QDialog):
	"""
	Dialogue for adding a user to the contact list.
	Provides the user with a list of possible contacts and
	adds the selected one to contacts.
	"""
	
	def __init__(self, transport, database):
		super().__init__()
		self.transport = transport
		self.database = database
		
		self.setFixedSize(350, 120)
		self.setWindowTitle('Выберите контакт для добавления:')
		self.setAttribute(Qt.WA_DeleteOnClose)
		self.setModal(True)
		
		self.selector_label = QLabel('Выберите контакт для добавления:', self)
		self.selector_label.setFixedSize(200, 20)
		self.selector_label.move(10, 0)
		
		self.selector = QComboBox(self)
		self.selector.setFixedSize(200, 20)
		self.selector.move(10, 30)
		
		self.btn_refresh = QPushButton('Обновить список', self)
		self.btn_refresh.setFixedSize(100, 30)
		self.btn_refresh.move(60, 60)
		
		self.btn_ok = QPushButton('Добавить', self)
		self.btn_ok.setFixedSize(100, 30)
		self.btn_ok.move(230, 20)
		
		self.btn_cancel = QPushButton('Отмена', self)
		self.btn_cancel.setFixedSize(100, 30)
		self.btn_cancel.move(230, 60)
		self.btn_cancel.clicked.connect(self.close)
		
		# Заполняем список возможных контактов
		self.possible_contacts_update()
		# Назначаем действие на кнопку обновить
		self.btn_refresh.clicked.connect(self.update_possible_contacts)
	
	def possible_contacts_update(self):
		"""
		Method for filling out the list of possible contacts.
		Creates a list of all registered users
		except for those already added to contacts and yourself.
		:return:
		"""
		self.selector.clear()
		# множества всех контактов и контактов клиента
		contacts_list = set(self.database.get_contacts())
		users_list = set(self.database.get_users())
		# Удалим сами себя из списка пользователей, чтобы нельзя было добавить
		# самого себя
		users_list.remove(self.transport.username)
		# Добавляем список возможных контактов
		self.selector.addItems(users_list - contacts_list)
	
	def update_possible_contacts(self):
		"""
		Method for updating the list of possible contacts. Requests from the server
		list of known users and blots the contents of the window.
		:return:
		"""
		try:
			self.transport.user_list_update()
		except OSError:
			pass
		else:
			client_log.debug('Обновление списка пользователей с сервера выполнено')
			self.possible_contacts_update()



from PyQt5.QtWidgets import QDialog, QPushButton, QLineEdit, QApplication, QLabel, qApp


class UserNameDialog(QDialog):
	"""
	A class that implements a start dialog with a login and password request
	user.
	"""
	def __init__(self):
		super().__init__()
		
		self.ok_pressed = False
		
		self.setWindowTitle('Привет!')
		self.setFixedSize(175, 135)
		
		self.label = QLabel('Введите имя пользователя:', self)
		self.label.move(10, 10)
		self.label.setFixedSize(150, 10)
		
		self.client_name = QLineEdit(self)
		self.client_name.setFixedSize(154, 20)
		self.client_name.move(10, 30)
		
		self.btn_ok = QPushButton('Начать', self)
		self.btn_ok.move(10, 105)
		self.btn_ok.clicked.connect(self.click)
		
		self.btn_cancel = QPushButton('Выход', self)
		self.btn_cancel.move(90, 105)
		self.btn_cancel.clicked.connect(qApp.exit)
		
		self.label_passwd = QLabel('Введите пароль:', self)
		self.label_passwd.move(10, 55)
		self.label_passwd.setFixedSize(150, 15)
		
		self.client_passwd = QLineEdit(self)
		self.client_passwd.setFixedSize(154, 20)
		self.client_passwd.move(10, 75)
		self.client_passwd.setEchoMode(QLineEdit.Password)
		
		self.show()
	
	def click(self):
		"""
		OK button handler method
		:return:
		"""
		if self.client_name.text() and self.client_passwd.text():
			self.ok_pressed = True
			qApp.exit()


if __name__ == '__main__':
	app = QApplication([])
	dial = UserNameDialog()
	app.exec_()



import base64
import json

from Cryptodome.Cipher import PKCS1_OAEP
from Cryptodome.PublicKey import RSA
from PyQt5.QtCore import pyqtSlot, Qt
from PyQt5.QtGui import QStandardItemModel, QStandardItem, QBrush, QColor
from PyQt5.QtWidgets import QMainWindow, qApp, QMessageBox

from chatclient.сhatclient.Log.client_log_config import client_log
from chatclient.сhatclient.Mainlib.exception import ServerError
from chatclient.сhatclient.Mainlib.variables import MESSAGE_TEXT, SENDER
from chatclient.сhatclient.client.add_contact_window import AddContactDialog
from chatclient.сhatclient.client.client_main_window_conv import Ui_MainClientWindow
from chatclient.сhatclient.client.del_contact_window import DelContactDialog


class ClientMainWindow(QMainWindow):
	"""
	Class is the main user window.
    Contains all the main logic of the client module.
    The window configuration is created in QTDesigner and loaded from
    converted file main_window_conv.py
	"""
	
	def __init__(self, database, transport, keys):
		super().__init__()
		# основные переменные
		self.database = database
		self.transport = transport
		
		# объект - дешифорвщик сообщений с предзагруженным ключём
		self.decrypter = PKCS1_OAEP.new(keys)
		
		# Загружаем конфигурацию окна из дизайнера
		self.ui = Ui_MainClientWindow()
		self.ui.setupUi(self)
		
		# Кнопка "Выход"
		self.ui.menu_exit.triggered.connect(qApp.exit)
		
		# Кнопка отправить сообщение
		self.ui.btn_send.clicked.connect(self.send_message)
		
		# "добавить контакт"
		self.ui.btn_add_contact.clicked.connect(self.add_contact_window)
		self.ui.menu_add_contact.triggered.connect(self.add_contact_window)
		
		# Удалить контакт
		self.ui.btn_remove_contact.clicked.connect(self.delete_contact_window)
		self.ui.menu_del_contact.triggered.connect(self.delete_contact_window)
		
		# Дополнительные требующиеся атрибуты
		self.contacts_model = None
		self.history_model = None
		self.messages = QMessageBox()
		self.current_chat = None
		self.current_chat_key = None
		self.encryptor = None
		self.ui.list_messages.setHorizontalScrollBarPolicy(
			Qt.ScrollBarAlwaysOff)
		self.ui.list_messages.setWordWrap(True)
		
		# Даблклик по листу контактов отправляется в обработчик
		self.ui.list_contacts.doubleClicked.connect(self.select_active_user)
		
		self.clients_list_update()
		self.set_disabled_input()
		self.show()
	
	def set_disabled_input(self):
		"""
		Method making input fields inactive
		:return:
		"""
		# Надпись  - получатель.
		self.ui.label_new_message.setText(
			'Для выбора получателя дважды кликните на нем в окне контактов.')
		self.ui.text_message.clear()
		if self.history_model:
			self.history_model.clear()
		
		# Поле ввода и кнопка отправки неактивны до выбора получателя.
		self.ui.btn_clear.setDisabled(True)
		self.ui.btn_send.setDisabled(True)
		self.ui.text_message.setDisabled(True)
		
		self.encryptor = None
		self.current_chat = None
		self.current_chat_key = None
	
	def history_list_update(self):
		"""
		The filling method of the corresponding QListView
        history of correspondence with the current interlocutor.
		:return:
		"""
		# Получаем историю сортированную по дате
		list = sorted(
			self.database.get_history(
				self.current_chat),
			key=lambda item: item[3])
		# Если модель не создана, создадим.
		if not self.history_model:
			self.history_model = QStandardItemModel()
			self.ui.list_messages.setModel(self.history_model)
		# Очистим от старых записей
		self.history_model.clear()
		# Берём не более 20 последних записей.
		length = len(list)
		start_index = 0
		if length > 20:
			start_index = length - 20
		# Заполнение модели записями, так-же стоит разделить входящие
		# и исходящие выравниванием и разным фоном.
		# отображает только последие 20 сообщений
		for i in range(start_index, length):
			item = list[i]
			if item[1] == 'in':
				mess = QStandardItem(
					f'Входящее от {item[3].replace(microsecond=0)}:\n {item[2]}')
				mess.setEditable(False)
				mess.setBackground(QBrush(QColor(255, 213, 213)))
				mess.setTextAlignment(Qt.AlignLeft)
				self.history_model.appendRow(mess)
			else:
				mess = QStandardItem(
					f'Исходящее от {item[3].replace(microsecond=0)}:\n {item[2]}')
				mess.setEditable(False)
				mess.setTextAlignment(Qt.AlignRight)
				mess.setBackground(QBrush(QColor(204, 255, 204)))
				self.history_model.appendRow(mess)
		self.ui.list_messages.scrollToBottom()
	
	def select_active_user(self):
		"""
		Double click event handler method on the contact list
		:return:
		"""
		# Выбранный пользователем (даблклик) находится в выделеном элементе в
		# QListView
		# online_users_list = ServerStorage.active_users_list() # получаем активных пользователей в
		# # списке кортежей из базы сервера
		# self.current_chat = self.ui.list_contacts.currentIndex().data()
		# if self.current_chat in online_users_list.count():  # проверяем список активных пользователей
		#     self.set_active_user()  # вызываем основную функцию
		# else:
		#     self.messages.warning(
		#         self, 'Ошибка', 'Данный пользователь не в сети.')
		
		self.current_chat = self.ui.list_contacts.currentIndex().data()
		online_users_list = self.transport.online_users_list_request()  # получаем активных пользователей в
		# списке кортежей из базы сервера
		if self.current_chat in online_users_list:
			self.set_active_user()  # вызываем основную функцию
		else:
			self.messages.warning(self, 'Ошибка', 'Данный пользователь не в сети.')
	
	def set_active_user(self):
		"""
		A method for activating a chat with an interlocutor.
		:return:
		"""
		# Запрашиваем публичный ключ пользователя и создаём объект шифрования
		try:
			self.current_chat_key = self.transport.key_request(
				self.current_chat)
			client_log.debug(f'Загружен открытый ключ для {self.current_chat}')
			if self.current_chat_key:
				self.encryptor = PKCS1_OAEP.new(
					RSA.import_key(self.current_chat_key))
		except (OSError, json.JSONDecodeError):
			self.current_chat_key = None
			self.encryptor = None
			client_log.debug(f'Не удалось получить ключ для {self.current_chat}')
		
		# Если ключа нет то ошибка, что не удалось начать чат с пользователем
		if not self.current_chat_key:
			self.messages.warning(
				self, 'Ошибка', 'Для выбранного пользователя нет ключа шифрования.')
			return
		
		# Ставим надпись и активируем кнопки
		self.ui.label_new_message.setText(
			f'Введите сообщенние для {self.current_chat}:')
		self.ui.btn_clear.setDisabled(False)
		self.ui.btn_send.setDisabled(False)
		self.ui.text_message.setDisabled(False)
		
		# Заполняем окно историю сообщений по требуемому пользователю.
		self.history_list_update()
	
	def clients_list_update(self):
		"""
		Method for updating the contact list.
		:return:
		"""
		contacts_list = self.database.get_contacts()
		self.contacts_model = QStandardItemModel()
		for i in sorted(contacts_list):
			item = QStandardItem(i)
			item.setEditable(False)
			self.contacts_model.appendRow(item)
		self.ui.list_contacts.setModel(self.contacts_model)
	
	def add_contact_window(self):
		"""
		Method creating a window - a dialog for adding a contact
		:return:
		"""
		global select_dialog
		select_dialog = AddContactDialog(self.transport, self.database)
		select_dialog.btn_ok.clicked.connect(
			lambda: self.add_contact_action(select_dialog))
		select_dialog.show()
	
	def add_contact_action(self, item):
		"""
		Method to handle clicking the 'Add' button
		:param item:
		:return:
		"""
		new_contact = item.selector.currentText()
		self.add_contact(new_contact)
		item.close()
	
	def add_contact(self, new_contact):
		"""
		Method adding contact to server and client BD.
        After updating the databases, the contents of the window are also updated.
		:param new_contact:
		:return:
		"""
		try:
			self.transport.add_contact(new_contact)
		except ServerError as err:
			self.messages.critical(self, 'Ошибка сервера', err.text)
		except OSError as err:
			if err.errno:
				self.messages.critical(
					self, 'Ошибка', 'Потеряно соединение с сервером!')
				self.close()
			self.messages.critical(self, 'Ошибка', 'Таймаут соединения!')
		else:
			self.database.add_contact(new_contact)
			new_contact = QStandardItem(new_contact)
			new_contact.setEditable(False)
			self.contacts_model.appendRow(new_contact)
			client_log.info(f'Успешно добавлен контакт {new_contact}')
			self.messages.information(
				self, 'Успех', 'Контакт успешно добавлен.')
	
	def delete_contact_window(self):
		"""
		A method that creates a window for deleting a contact.
		:return:
		"""
		global remove_dialog
		remove_dialog = DelContactDialog(self.database)
		remove_dialog.btn_ok.clicked.connect(
			lambda: self.delete_contact(remove_dialog))
		remove_dialog.show()
	
	def delete_contact(self, item):
		"""
		Method for removing contact from server and client BD.
        After updating the databases, the contents of the window are also updated.
		:param item:
		:return:
		"""
		selected = item.selector.currentText()
		try:
			self.transport.remove_contact(selected)
		except ServerError as err:
			self.messages.critical(self, 'Ошибка сервера', err.text)
		except OSError as err:
			if err.errno:
				self.messages.critical(
					self, 'Ошибка', 'Потеряно соединение с сервером!')
				self.close()
			self.messages.critical(self, 'Ошибка', 'Таймаут соединения!')
		else:
			self.database.del_contact(selected)
			self.clients_list_update()
			client_log.info(f'Успешно удалён контакт {selected}')
			self.messages.information(self, 'Успех', 'Контакт успешно удалён.')
			item.close()
			# Если удалён активный пользователь, то деактивируем поля ввода.
			if selected == self.current_chat:
				self.current_chat = None
				self.set_disabled_input()
	
	def send_message(self):
		"""
		The function of sending a message to the current interlocutor.
		Implements message encryption and sending.
		"""
		# Текст в поле, проверяем что поле не пустое затем забирается сообщение
		# и поле очищается
		message_text = self.ui.text_message.toPlainText()
		self.ui.text_message.clear()
		if not message_text:
			return
		# Шифруем сообщение ключом получателя и упаковываем в base64.
		message_text_encrypted = self.encryptor.encrypt(
			message_text.encode('utf8'))
		message_text_encrypted_base64 = base64.b64encode(
			message_text_encrypted)
		try:
			self.transport.send_message(
				self.current_chat,
				message_text_encrypted_base64.decode('ascii'))
			pass
		except ServerError as err:
			self.messages.critical(self, 'Ошибка', err.text)
		except OSError as err:
			if err.errno:
				self.messages.critical(
					self, 'Ошибка', 'Потеряно соединение с сервером!')
				self.close()
			self.messages.critical(self, 'Ошибка', 'Таймаут соединения!')
		except (ConnectionResetError, ConnectionAbortedError):
			self.messages.critical(
				self, 'Ошибка', 'Потеряно соединение с сервером!')
			self.close()
		else:
			self.database.save_message(self.current_chat, 'out', message_text)
			client_log.debug(
				f'Отправлено сообщение для {self.current_chat}: {message_text}')
			self.history_list_update()
	
	@pyqtSlot(dict)
	def message(self, message):
		"""
		Slot handler of incoming messages, performs decryption
        received messages and their saving in the message history.
        Asks the user if the message is not from the current one
        interlocutor. Changes the interlocutor if necessary.
		:param message:
		:return:
		"""
		# Получаем строку байтов
		encrypted_message = base64.b64decode(message[MESSAGE_TEXT])
		# Декодируем строку, при ошибке выдаём сообщение и завершаем функцию
		try:
			decrypted_message = self.decrypter.decrypt(encrypted_message)
		except (ValueError, TypeError):
			self.messages.warning(
				self, 'Ошибка', 'Не удалось декодировать сообщение.')
			return
		# Сохраняем сообщение в базу и обновляем историю сообщений или
		# открываем новый чат.
		self.database.save_message(
			self.current_chat,
			'in',
			decrypted_message.decode('utf8'))
		
		sender = message[SENDER]
		if sender == self.current_chat:
			self.history_list_update()
		else:
			# Проверим есть ли такой пользователь у нас в контактах:
			if self.database.check_contact(sender):
				# Если есть, спрашиваем и желании открыть с ним чат и открываем
				# при желании
				if self.messages.question(
						self,
						'Новое сообщение',
						f'Получено новое сообщение от {sender}, открыть чат с ним?',
						QMessageBox.Yes,
						QMessageBox.No) == QMessageBox.Yes:
					self.current_chat = sender
					self.set_active_user()
			else:
				print('NO')
				# Раз нету,спрашиваем хотим ли добавить юзера в контакты.
				if self.messages.question(
						self,
						'Новое сообщение',
						f'Получено новое сообщение от {sender}.\n Данного пользователя нет в вашем контакт-листе.\n '
						f'Добавить в контакты и открыть чат с ним?',
						QMessageBox.Yes,
						QMessageBox.No) == QMessageBox.Yes:
					self.add_contact(sender)
					self.current_chat = sender
					# Нужно заново сохранить сообщение, иначе оно будет потеряно,
					# т.к. на момент предыдущего вызова контакта не было.
					self.database.save_message(
						self.current_chat, 'in', decrypted_message.decode('utf8'))
					self.set_active_user()
	
	@pyqtSlot()
	def connection_lost(self):
		"""
		Slot handler for lost connection to the server.
        Shows a warning window and exits the application.
		:return:
		"""
		self.messages.warning(
			self,
			'Сбой соединения',
			'Потеряно соединение с сервером. ')
		self.close()
	
	@pyqtSlot()
	def sig_205(self):
		"""
		Slot performing database update on server command
		"""
		if self.current_chat and not self.database.check_user(
				self.current_chat):
			self.messages.warning(
				self,
				'Сочувствую',
				'К сожалению собеседник был удалён с сервера.')
			self.set_disabled_input()
			self.current_chat = None
		self.clients_list_update()
	
	def make_connection(self, trans_obj):
		"""
		Method for connecting signals and slots
		:param trans_obj:
		:return:
		"""
		trans_obj.new_message.connect(self.message)
		trans_obj.connection_lost.connect(self.connection_lost)
		trans_obj.message_205.connect(self.sig_205)



from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QDialog, QLabel, QComboBox, QPushButton, QApplication


class DelContactDialog(QDialog):
	"""
	Dialogue for deleting a contact. Maintains the current contact list,
	does not have handlers for actions.
	"""
	def __init__(self, database):
		super().__init__()
		self.database = database
		
		self.setFixedSize(350, 120)
		self.setWindowTitle('Выберите контакт для удаления:')
		self.setAttribute(Qt.WA_DeleteOnClose)
		self.setModal(True)
		
		self.selector_label = QLabel('Выберите контакт для удаления:', self)
		self.selector_label.setFixedSize(200, 20)
		self.selector_label.move(10, 0)
		
		self.selector = QComboBox(self)
		self.selector.setFixedSize(200, 20)
		self.selector.move(10, 30)
		
		self.btn_ok = QPushButton('Удалить', self)
		self.btn_ok.setFixedSize(100, 30)
		self.btn_ok.move(230, 20)
		
		self.btn_cancel = QPushButton('Отмена', self)
		self.btn_cancel.setFixedSize(100, 30)
		self.btn_cancel.move(230, 60)
		self.btn_cancel.clicked.connect(self.close)
		
		# заполнитель контактов для удаления
		self.selector.addItems(sorted(self.database.get_contacts()))


if __name__ == '__main__':
	app = QApplication([])
	window = DelContactDialog(None)
	window.show()
	app.exec_()



import binascii
import hashlib
import hmac
import json
import socket
import threading
import time

from PyQt5.QtCore import pyqtSignal, QObject


from chatclient.сhatclient.Log.client_log_config import client_log
from chatclient.сhatclient.Mainlib.JIM import send_message, get_message
from chatclient.сhatclient.Mainlib.exception import ServerError
from chatclient.сhatclient.Mainlib.variables import ACTION, PRESENCE, TIME, USER, ACCOUNT_NAME, PUBLIC_KEY, RESPONSE, \
	ERROR, DATA, RESPONSE_511, MESSAGE, SENDER, DESTINATION, MESSAGE_TEXT, GET_CONTACTS, LIST_INFO, USERS_REQUEST, \
	PUBLIC_KEY_REQUEST, ONLINE_USERS_REQUEST, ADD_CONTACT, REMOVE_CONTACT, EXIT

# Объект блокировки для работы с сокетом.
socket_lock = threading.Lock()


class ClientTransport(threading.Thread, QObject):
	"""
	The class that implements the transport subsystem of the client
	module. Responsible for interacting with the server.
	"""
	# Сигналы новое сообщение и потеря соединения
	new_message = pyqtSignal(dict)
	message_205 = pyqtSignal()
	connection_lost = pyqtSignal()
	
	def __init__(self, port, ip_address, database, username, passwd, keys):
		# Вызываем конструкторы предков
		threading.Thread.__init__(self)
		QObject.__init__(self)
		
		# Класс База данных - работа с базой
		self.database = database
		# Имя пользователя
		self.username = username
		# Пароль
		self.password = passwd
		# Сокет для работы с сервером
		self.transport = None
		# Набор ключей для шифрования
		self.keys = keys
		# Устанавливаем соединение:
		self.connection_init(port, ip_address)
		# Обновляем таблицы известных пользователей и контактов
		try:
			self.user_list_update()
			self.contacts_list_update()
		except OSError as err:
			if err.errno:
				client_log.critical(f'Потеряно соединение с сервером.')
				raise ServerError('Потеряно соединение с сервером!')
			client_log.error(
				'Timeout соединения при обновлении списков пользователей.')
		except json.JSONDecodeError:
			client_log.critical(f'Потеряно соединение с сервером.')
			raise ServerError('Потеряно соединение с сервером!')
		# Флаг продолжения работы транспорта.
		self.running = True
	
	def connection_init(self, port, ip):
		"""
		Method responsible for establishing a connection to the server
		:param port:
		:param ip:
		:return:
		"""
		# Инициализация сокета и сообщение серверу о нашем появлении
		self.transport = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		
		# Таймаут необходим для освобождения сокета.
		self.transport.settimeout(5)
		
		# Соединяемся, 5 попыток соединения, флаг успеха ставим в True если
		# удалось
		connected = False
		for i in range(5):
			client_log.info(f'Попытка подключения №{i + 1}')
			try:
				self.transport.connect((ip, port))
			except (OSError, ConnectionRefusedError):
				pass
			else:
				connected = True
				client_log.debug("Connection established.")
				break
			time.sleep(1)
		
		# Если соединится не удалось - исключение
		if not connected:
			client_log.critical('Не удалось установить соединение с сервером')
			raise ServerError('Не удалось установить соединение с сервером')
		
		client_log.debug('Starting auth dialog.')
		
		# Запускаем процедуру авторизации
		# Получаем хэш пароля
		passwd_bytes = self.password.encode('utf-8')
		salt = self.username.lower().encode('utf-8')
		passwd_hash = hashlib.pbkdf2_hmac('sha512', passwd_bytes, salt, 10000)
		passwd_hash_string = binascii.hexlify(passwd_hash)
		
		client_log.debug(f'Passwd hash ready: {passwd_hash_string}')
		
		# Получаем публичный ключ и декодируем его из байтов
		pubkey = self.keys.publickey().export_key().decode('ascii')
		
		# Авторизируемся на сервере
		with socket_lock:
			presense = {
				ACTION: PRESENCE,
				TIME: time.time(),
				USER: {
					ACCOUNT_NAME: self.username,
					PUBLIC_KEY: pubkey
				}
			}
			client_log.debug(f"Presense message = {presense}")
			# Отправляем серверу приветственное сообщение.
			try:
				send_message(self.transport, presense)
				ans = get_message(self.transport)
				client_log.debug(f'Server response = {ans}.')
				# Если сервер вернул ошибку, бросаем исключение.
				if RESPONSE in ans:
					if ans[RESPONSE] == 400:
						raise ServerError(ans[ERROR])
					elif ans[RESPONSE] == 511:
						# Если всё нормально, то продолжаем процедуру
						# авторизации.
						ans_data = ans[DATA]
						hash = hmac.new(passwd_hash_string, ans_data.encode('utf-8'), 'MD5')
						digest = hash.digest()
						my_ans = RESPONSE_511
						my_ans[DATA] = binascii.b2a_base64(
							digest).decode('ascii')
						send_message(self.transport, my_ans)
						self.process_server_ans(get_message(self.transport))
			except (OSError, json.JSONDecodeError) as err:
				client_log.debug(f'Connection error.', exc_info=err)
				raise ServerError('Сбой соединения в процессе авторизации.')
	
	def process_server_ans(self, message):
		"""
		Method to handle incoming messages from the server
		:param message:
		:return:
		"""
		client_log.debug(f'Разбор сообщения от сервера: {message}')
		
		# Если это подтверждение чего-либо
		if RESPONSE in message:
			if message[RESPONSE] == 200 or 202:
				return
			elif message[RESPONSE] == 400:
				raise ServerError(f'{message[ERROR]}')
			elif message[RESPONSE] == 205:
				self.user_list_update()
				self.contacts_list_update()
				self.message_205.emit()
			else:
				client_log.error(
					f'Принят неизвестный код подтверждения {message[RESPONSE]}')
		
		# Если это сообщение от пользователя добавляем в базу, даём сигнал о
		# новом сообщении
		elif ACTION in message and message[ACTION] == MESSAGE and SENDER in message and DESTINATION in message \
				and MESSAGE_TEXT in message and message[DESTINATION] == self.username:
			client_log.debug(
				f'Получено сообщение от пользователя {message[SENDER]}:{message[MESSAGE_TEXT]}')
			self.new_message.emit(message)
	
	def contacts_list_update(self):
		"""
		Method updating the list of contacts from the server.
		:return:
		"""
		self.database.contacts_clear()
		client_log.debug(f'Запрос контакт листа для пользователся {self.name}')
		req = {
			ACTION: GET_CONTACTS,
			TIME: time.time(),
			USER: self.username
		}
		client_log.debug(f'Сформирован запрос {req}')
		with socket_lock:
			send_message(self.transport, req)
			ans = get_message(self.transport)
		client_log.debug(f'Получен ответ {ans}')
		if RESPONSE in ans and ans[RESPONSE] == 202:
			for contact in ans[LIST_INFO]:
				self.database.add_contact(contact)
		else:
			client_log.error('Не удалось обновить список контактов.')
	
	def user_list_update(self):
		"""
		Method that updates the list of users from the server.
		:return:
		"""
		client_log.debug(f'Запрос списка известных пользователей {self.username}')
		req = {
			ACTION: USERS_REQUEST,
			TIME: time.time(),
			ACCOUNT_NAME: self.username
		}
		with socket_lock:
			send_message(self.transport, req)
			ans = get_message(self.transport)
		if RESPONSE in ans and ans[RESPONSE] == 202:
			self.database.add_users(ans[LIST_INFO])
		else:
			client_log.error('Не удалось обновить список известных пользователей.')
	
	def key_request(self, user):
		"""
		Method that requests the public key of the user from the server
		:param user:
		:return:
		"""
		client_log.debug(f'Запрос публичного ключа для {user}')
		req = {
			ACTION: PUBLIC_KEY_REQUEST,
			TIME: time.time(),
			ACCOUNT_NAME: user
		}
		with socket_lock:
			send_message(self.transport, req)
			ans = get_message(self.transport)
		if RESPONSE in ans and ans[RESPONSE] == 511:
			return ans[DATA]
		else:
			client_log.error(f'Не удалось получить ключ собеседника{user}.')
	
	def online_users_list_request(self):
		req = {
			ACTION: ONLINE_USERS_REQUEST,
			TIME: time.time()
		}
		with socket_lock:
			send_message(self.transport, req)
			self.process_server_ans(get_message(self.transport))
	
	def add_contact(self, contact):
		"""
		A method that sends information about adding a contact to the server.
		:param contact:
		:return:
		"""
		client_log.debug(f'Создание контакта {contact}')
		req = {
			ACTION: ADD_CONTACT,
			TIME: time.time(),
			USER: self.username,
			ACCOUNT_NAME: contact
		}
		with socket_lock:
			send_message(self.transport, req)
			self.process_server_ans(get_message(self.transport))
	
	def remove_contact(self, contact):
		"""
		Method that sends information about deleting a contact to the server
		:param contact:
		:return:
		"""
		client_log.debug(f'Удаление контакта {contact}')
		req = {
			ACTION: REMOVE_CONTACT,
			TIME: time.time(),
			USER: self.username,
			ACCOUNT_NAME: contact
		}
		with socket_lock:
			send_message(self.transport, req)
			self.process_server_ans(get_message(self.transport))
	
	def transport_shutdown(self):
		"""
		Method notifying the server when the client is finished
		:return:
		"""
		self.running = False
		message = {
			ACTION: EXIT,
			TIME: time.time(),
			ACCOUNT_NAME: self.username
		}
		with socket_lock:
			try:
				send_message(self.transport, message)
			except OSError:
				pass
		client_log.debug('Транспорт завершает работу.')
		time.sleep(0.5)
	
	def send_message(self, to, message):
		"""
		Method sending messages to the server for the user
		:param to:
		:param message:
		:return:
		"""
		message_dict = {
			ACTION: MESSAGE,
			SENDER: self.username,
			DESTINATION: to,
			TIME: time.time(),
			MESSAGE_TEXT: message
		}
		client_log.debug(f'Сформирован словарь сообщения: {message_dict}')
		# Необходимо дождаться освобождения сокета для отправки сообщения
		with socket_lock:
			send_message(self.transport, message_dict)
			self.process_server_ans(get_message(self.transport))
			client_log.info(f'Отправлено сообщение для пользователя {to}')
	
	def run(self):
		"""
		Method containing the main cycle of the transport stream
		:return:
		"""
		client_log.debug('Запущен процесс - приёмник собщений с сервера.')
		while self.running:
			# Отдыхаем секунду и снова пробуем захватить сокет.
			# если не сделать тут задержку, то отправка может достаточно долго
			# ждать освобождения сокета.
			time.sleep(1)
			message = None
			with socket_lock:
				try:
					self.transport.settimeout(0.5)
					message = get_message(self.transport)
				except OSError as err:
					if err.errno:
						client_log.critical(f'Потеряно соединение с сервером.')
						self.running = False
						self.connection_lost.emit()
				# Проблемы с соединением
				except (ConnectionError, ConnectionAbortedError, ConnectionResetError, json.JSONDecodeError, TypeError):
					client_log.debug(f'Потеряно соединение с сервером.')
					self.running = False
					self.connection_lost.emit()
				finally:
					self.transport.settimeout(5)
			
			# Если сообщение получено, то вызываем функцию обработчик:
			if message:
				client_log.debug(f'Принято сообщение с сервера: {message}')
				self.process_server_ans(message)



from setuptools import setup, find_packages

setup(name="My_Learn_Messenger_Client",
      version="0.1.1",
      description="Learn_GB_Messenger_Client",
      author="Nikolai Nikolaev by GB",
      author_email="nikolaeff2007@mail.ru",
      packages=find_packages(),
      install_requires=['PyQt5', 'sqlalchemy', 'pycryptodome', 'pycryptodomex']
      )


# -*- coding: utf-8 -*-
import argparse
import os
import sys

from Cryptodome.PublicKey import RSA
from PyQt5.QtWidgets import QApplication, QMessageBox

from chatclient.сhatclient.Log.Log_decorators import log

from chatclient.сhatclient.Log.client_log_config import client_log
from chatclient.сhatclient.Mainlib.exception import ServerError
from chatclient.сhatclient.client.Client_DataBase import ClientDatabase
from chatclient.сhatclient.client.client_main_window import ClientMainWindow
from chatclient.сhatclient.client.start_dialog_window import UserNameDialog
from chatclient.сhatclient.client.transport import ClientTransport


# Парсер аргументов коммандной строки

@log
def arg_parser():
	"""
    Parser for command line arguments, returns a tuple of 4 elements
    server address, port, username, password.
	"""
	parser = argparse.ArgumentParser()
	parser.add_argument('addr', default='127.0.0.1', nargs='?')
	parser.add_argument('port', default=7777, type=int, nargs='?')
	parser.add_argument('-n', '--name', default=None, nargs='?')
	parser.add_argument('-p', '--password', default='', nargs='?')
	namespace = parser.parse_args(sys.argv[1:])
	server_address = namespace.addr
	server_port = namespace.port
	client_name = namespace.name
	client_passwd = namespace.password
	
	return server_address, server_port, client_name, client_passwd


# Основная функция клиента
if __name__ == '__main__':
	# Загружаем параметы коммандной строки
	server_address, server_port, client_name, client_passwd = arg_parser()
	client_log.debug('Args loaded')
	
	# Создаём клиентское приложение
	client_app = QApplication(sys.argv)
	
	# Если имя пользователя не было указано в командной строке то запросим его
	start_dialog = UserNameDialog()
	if not client_name or not client_passwd:
		client_app.exec_()
		# Если пользователь ввёл имя и нажал ОК, то сохраняем введённое и
		# удаляем объект, иначе выходим
		if start_dialog.ok_pressed:
			client_name = start_dialog.client_name.text()
			client_passwd = start_dialog.client_passwd.text()
			client_log.debug(f'Using USERNAME = {client_name}, PASSWD = {client_passwd}.')
		else:
			sys.exit(0)
	
	# Записываем логи
	client_log.info(
		f'Запущен клиент с параметрами: адрес сервера: {server_address} , порт: {server_port}, имя пользователя: {client_name}')
	
	# Загружаем ключи с файла, если же файла нет, то генерируем новую пару.
	dir_path = os.getcwd()
	key_file = os.path.join(dir_path, f'{client_name}.key')
	if not os.path.exists(key_file):
		keys = RSA.generate(2048, os.urandom)
		with open(key_file, 'wb') as key:
			key.write(keys.export_key())
	else:
		with open(key_file, 'rb') as key:
			keys = RSA.import_key(key.read())
	
	# !!!keys.publickey().export_key()
	client_log.debug("Keys sucsessfully loaded.")
	# Создаём объект базы данных
	database = ClientDatabase(client_name)
	# Создаём объект - транспорт и запускаем транспортный поток
	try:
		transport = ClientTransport(
			server_port,
			server_address,
			database,
			client_name,
			client_passwd,
			keys)
		client_log.debug("Transport ready.")
	except ServerError as error:
		message = QMessageBox()
		message.critical(start_dialog, 'Ошибка сервера', error.text)
		sys.exit(1)
	transport.setDaemon(True)
	transport.start()
	
	# Удалим объект диалога за ненадобностью
	del start_dialog
	
	# Создаём GUI
	main_window = ClientMainWindow(database, transport, keys)
	main_window.make_connection(transport)
	main_window.setWindowTitle(f'Чат Клиент alpha release - {client_name}')
	client_app.exec_()
	
	# Раз графическая оболочка закрылась, закрываем транспорт
	transport.transport_shutdown()
	transport.join()
